#!/usr/bin/env python
#
# duo_openvpn_auth.py
# OpenVPN 2.3 Static Challenge integration example auth script for use with
# Viscosity 1.4.5+ (Mac) or Viscosity 1.4.6+ (Windows).
# Usage: auth-user-pass-verify /path/to/duo_openvpn_auth.py via-env
# http://www.sparklabs.com/viscosity/
#

## pfSense Python Path
##!/usr/local/bin/python

#------------------------------------------------------------------------------
# VARIABLES
#------------------------------------------------------------------------------

# The path to the normal OpenVPN authentication script to do username/password
# authentication (for example a PAM script, LDAP script, etc.)
# E.g. user_pass_auth_script = '/var/etc/openvpn/server1.php'
user_pass_auth_script = ''

# Your Duo Security Auth API details (Integration key, Secret key, and API hostname)
duo_ikey = ''
duo_skey = ''
duo_host = 'api-xxxxxxxx.duosecurity.com'

#------------------------------------------------------------------------------

import sys, os
from base64 import b64decode

# Read in the username and password from the environment
username = os.environ['username']
password = os.environ['password']
ipAddr = os.environ['untrusted_ip']

# Extract the token from the encoded password
if password.startswith('SCRV1:'):
	# Extract the actual password and token
	passwordSplit = password.split(':')
	password = b64decode(passwordSplit[1])
	token = b64decode(passwordSplit[2])
else:
	# Invalid data
	sys.exit(1)

# Check the username and password against the standard authentication script
os.environ['password'] = password
import subprocess
try:
	subprocess.check_call(user_pass_auth_script)
	loginValid = True
except:
	loginValid = False

if not loginValid:
	sys.exit(1)

# Check with Duo Security whether the client can authenticate (or by pass auth). Clients
# that need to enroll are not supported (they can enroll via web, email, etc.)
import duo_client
duoAuth = duo_client.Auth(duo_ikey, duo_skey, duo_host)
response = duoAuth.preauth(username, None, ipAddr)
if not response['result'] in ('allow', 'auth'):
	# User is not permitted or not enrolled
	sys.exit(1)

# Authenticate the user
if response['result'] == 'allow':
	sys.exit(0)
if token in ('push', 'phone', 'sms'):
	response = duoAuth.auth(factor=token,
							username=username,
							ipaddr=ipAddr,
							device='auto')
else:
	response = duoAuth.auth(factor='auto',
							username=username,
							ipaddr=ipAddr)
if response['result'] == 'allow':
	sys.exit(0)					

sys.exit(1)
